/**
 * Functions/Methods that depend on the current runtime to function
 */
import { logError } from './log.ts';
import { process } from './node.ts';
import { RunTimeType } from './runtime.ts';
import { IRuntime, ITestBase } from '../types.ts';

let scrollRuntime: IRuntime | null = null;

/**
 * Kill program, displaying an error message
 * @param s
 */
export function die(s: string | Error): void {
	logError(s);
	process.stdin.setRawMode(false);
	console.error(s);
	getRuntime().then((r) => r.exit());
}

/**
 * Determine which Typescript runtime we are operating under
 */
export function runtimeType(): RunTimeType {
	let runtime = RunTimeType.Tsx;

	if ('Deno' in globalThis) {
		runtime = RunTimeType.Deno;
	} else if ('Bun' in globalThis) {
		runtime = RunTimeType.Bun;
	}

	return runtime;
}

/**
 * Get the adapter object for the current Runtime
 */
export async function getRuntime(): Promise<IRuntime> {
	if (scrollRuntime === null) {
		const runtime = runtimeType();
		const path = `../../${runtime}/mod.ts`;

		const pkg = await import(path);
		if ('default' in pkg) {
			scrollRuntime = pkg.default;
		}

		if (scrollRuntime !== null) {
			return Promise.resolve(scrollRuntime);
		}

		return Promise.reject('Missing default import');
	}

	return Promise.resolve(scrollRuntime);
}

/**
 * Get the common test interface object
 */
export async function getTestRunner(): Promise<ITestBase> {
	const runtime = runtimeType();
	const path = `../../${runtime}/test_base.ts`;
	const pkg = await import(path);
	if ('default' in pkg) {
		return pkg.default;
	}

	return pkg;
}
